/******************************************************************************
 * Copyright 2022-2023 TypeFox GmbH
 * This program and the accompanying materials are made available under the
 * terms of the MIT License, which is available in the project root.
 ******************************************************************************/
import { isDataType, isSimpleSelectStatement } from "./generated/ast.js";
import { toString } from "./sql-data-types.js";
import { CompletionItemKind } from 'vscode-languageserver';
import { getColumnCandidatesForSelectTableExpression } from "./sql-type-utilities.js";
import { isNamed } from "langium";
import { DefaultCompletionProvider } from "langium/lsp";
export class SqlCompletionProvider extends DefaultCompletionProvider {
    constructor(services) {
        super(services);
        this.completionOptions = {
            triggerCharacters: ['.']
        };
        this.dataTypes = services.dialect.dataTypes.allTypes();
    }
    getCompletion(document, params) {
        return super.getCompletion(document, params);
    }
    completionFor(context, next, acceptor) {
        var _a;
        if (isDataType(context.node) && next.property === 'dataTypeNames') {
            this.completeWithDataTypes(context, acceptor);
        }
        else if (isSimpleSelectStatement(context.node) && next.type === 'SelectElements') {
            const columnDescriptors = getColumnCandidatesForSelectTableExpression(context.node.$container);
            for (const columnDescriptor of columnDescriptors) {
                const text = (_a = columnDescriptor.name) !== null && _a !== void 0 ? _a : (isNamed(columnDescriptor.node) ? columnDescriptor.node.name : undefined);
                acceptor(context, {
                    kind: CompletionItemKind.Field,
                    label: text,
                    insertText: text,
                });
            }
        }
        return super.completionFor(context, next, acceptor);
    }
    completeWithDataTypes(context, acceptor) {
        for (const dataType of this.dataTypes) {
            acceptor(context, {
                label: toString(dataType),
                kind: CompletionItemKind.Class,
                insertText: toString(dataType),
            });
        }
    }
}
//# sourceMappingURL=sql-completion-provider.js.map