/******************************************************************************
 * Copyright 2022-2023 TypeFox GmbH
 * This program and the accompanying materials are made available under the
 * terms of the MIT License, which is available in the project root.
 ******************************************************************************/
import _ from "lodash";
export function isTypeANull(type) {
    return type.discriminator === "null";
}
export function isTypeABoolean(type) {
    return type.discriminator === "boolean";
}
export function isTypeANumber(type) {
    return [
        "integer",
        "real",
    ].includes(type.discriminator);
}
export function isTypeAReal(type) {
    return type.discriminator === "real";
}
export function isTypeADateTime(type) {
    return type.discriminator === "datetime";
}
export function isTypeAnInteger(type) {
    return type.discriminator === "integer";
}
export function isTypeARow(type) {
    return type.discriminator === "row";
}
export function isTypeAnEnum(type) {
    return type.discriminator === "enum";
}
export function isTypeABlob(type) {
    return type.discriminator === "blob";
}
export function isTypeAText(type) {
    return [
        "text",
    ].includes(type.discriminator);
}
export const Types = {
    Unknown: {
        discriminator: 'unknown'
    },
    Null: {
        discriminator: 'null',
    },
    ArrayOf(elementType) {
        return {
            discriminator: 'array',
            elementType
        };
    },
    Boolean: {
        discriminator: 'boolean'
    },
    Blob: {
        discriminator: 'blob'
    },
    Integer: {
        discriminator: 'integer'
    },
    Real: {
        discriminator: 'real'
    },
    DateTime: {
        discriminator: 'datetime'
    },
    Char(length = 100) {
        return {
            discriminator: 'text',
            length
        };
    },
    Enum(members) {
        return { discriminator: 'enum', members: members.filter(e => typeof e === 'string') };
    }
};
export function areTypesEqual(lhs, rhs) {
    return _.isEqual(lhs, rhs);
}
//# sourceMappingURL=sql-type-descriptors.js.map