/******************************************************************************
 * Copyright 2022-2023 TypeFox GmbH
 * This program and the accompanying materials are made available under the
 * terms of the MIT License, which is available in the project root.
 ******************************************************************************/
import _ from "lodash";
import { Types } from "./sql-type-descriptors.js";
const IntegerReturnsInteger = {
    operandType: Types.Integer,
    returnType: Types.Integer
};
const RealReturnsReal = {
    operandType: Types.Real,
    returnType: Types.Real
};
const BooleanReturnsBoolean = {
    operandType: Types.Boolean,
    returnType: Types.Boolean
};
const ReversedUnaryOperators = new Map();
ReversedUnaryOperators.set(IntegerReturnsInteger, ['+', '-']);
ReversedUnaryOperators.set(RealReturnsReal, ['+', '-']);
ReversedUnaryOperators.set(BooleanReturnsBoolean, ['NOT']);
const unaries = [...ReversedUnaryOperators.entries()].flatMap(([descr, operators]) => {
    return operators.map(op => [op, descr]);
});
function emptUnaryTable() {
    const result = {};
    const indices = ["+", "-", "NOT"];
    indices.forEach(op => result[op] = []);
    return result;
}
export const UnaryOperators = Object.entries(_.groupBy(unaries, ([op]) => op))
    .reduce((lhs, rhs) => {
    const key = rhs[0];
    lhs[key] = lhs[key].concat(rhs[1].map(descr => descr[1]));
    return lhs;
}, emptUnaryTable());
const IntegerIntegerReturnsInteger = {
    left: Types.Integer,
    right: Types.Integer,
    returnType: Types.Integer
};
const RealRealReturnsReal = {
    left: Types.Real,
    right: Types.Real,
    returnType: Types.Real
};
const IntegerIntegerReturnsBoolean = {
    left: Types.Integer,
    right: Types.Integer,
    returnType: Types.Boolean
};
const RealRealReturnsBoolean = {
    left: Types.Real,
    right: Types.Real,
    returnType: Types.Boolean
};
const BooleanBooleanReturnsBoolean = {
    left: Types.Boolean,
    right: Types.Boolean,
    returnType: Types.Boolean
};
const CharCharReturnsBoolean = {
    left: Types.Char(),
    right: Types.Char(),
    returnType: Types.Boolean
};
const CharCharReturnsChar = {
    left: Types.Char(),
    right: Types.Char(),
    returnType: Types.Char()
};
const CharCharReturnsReal = {
    left: Types.Char(),
    right: Types.Char(),
    returnType: Types.Real
};
const ReversedBinaryOperators = new Map();
ReversedBinaryOperators.set(IntegerIntegerReturnsInteger, ["%", "*", "+", "-", "/"]);
ReversedBinaryOperators.set(RealRealReturnsReal, ["%", "*", "+", "-", "/"]);
ReversedBinaryOperators.set(IntegerIntegerReturnsBoolean, ["<", "<=", "<>", "=", ">", ">="]);
ReversedBinaryOperators.set(RealRealReturnsBoolean, ["<", "<=", "<>", "=", ">", ">="]);
ReversedBinaryOperators.set(CharCharReturnsBoolean, ["<", "<=", "<>", "=", ">", ">=", "LIKE"]);
ReversedBinaryOperators.set(BooleanBooleanReturnsBoolean, ["<", "<=", "<>", "=", ">", ">=", "AND", "OR"]);
ReversedBinaryOperators.set(CharCharReturnsChar, ['||', 'LIKE', '::$', '::']);
ReversedBinaryOperators.set(CharCharReturnsReal, ['::%']);
const flatMap = [...ReversedBinaryOperators.entries()].flatMap(([descr, operators]) => {
    return operators.map(op => [op, descr]);
});
function emptyBinaryTable() {
    const result = {};
    const indices = ["%", "*", "+", "-", "/", "<", "<=", "<>", "=", ">", ">=", "AND", "OR", "||", "IS", "LIKE", "IN", "::", "::$", "::%"];
    indices.forEach(op => result[op] = []);
    return result;
}
export const BinaryOperators = Object.entries(_.groupBy(flatMap, ([op]) => op))
    .reduce((lhs, rhs) => {
    const key = rhs[0];
    lhs[key] = lhs[key].concat(rhs[1].map(descr => descr[1]));
    return lhs;
}, emptyBinaryTable());
//# sourceMappingURL=sql-type-operators.js.map